@@ -33,6 +33,9 @@
 #include <QHBoxLayout>
 #include <QFormLayout>
 #include <QPlainTextEdit>
+#include <QAction>
+#include <QMenu>
+#include <QClipboard>
 
 #include "ui_qgsattributesformproperties.h"
 #include "qgis_gui.h"
@@ -40,6 +43,7 @@
 #include "qgsexpressioncontextgenerator.h"
 #include "qgsattributeeditorelement.h"
 #include "qgspropertycollection.h"
+#include "qgsmessagebar.h"
 
 class QgsAttributesDnDTree;
 class QgsAttributeFormContainerEdit;
@@ -55,7 +59,6 @@ class GUI_EXPORT QgsAttributesFormProperties : public QWidget, public QgsExpress
     Q_OBJECT
 
   public:
-
     enum FieldPropertiesRoles
     {
       DnDTreeRole = Qt::UserRole,
@@ -65,33 +68,33 @@ class GUI_EXPORT QgsAttributesFormProperties : public QWidget, public QgsExpress
 
     struct RelationEditorConfiguration
     {
-      operator QVariant();
+        operator QVariant();
 
-      QString mRelationWidgetType;
-      QVariantMap mRelationWidgetConfig;
-      QVariant nmRelationId;
-      bool forceSuppressFormPopup = false;
-      QString label;
+        QString mRelationWidgetType;
+        QVariantMap mRelationWidgetConfig;
+        QVariant nmRelationId;
+        bool forceSuppressFormPopup = false;
+        QString label;
     };
 
     struct QmlElementEditorConfiguration
     {
-      QString qmlCode;
+        QString qmlCode;
     };
 
     struct HtmlElementEditorConfiguration
     {
-      QString htmlCode;
+        QString htmlCode;
     };
 
     struct TextElementEditorConfiguration
     {
-      QString text;
+        QString text;
     };
 
     struct SpacerElementEditorConfiguration
     {
-      bool drawLine = false;
+        bool drawLine = false;
     };
 
     /**
@@ -108,9 +111,9 @@ class GUI_EXPORT QgsAttributesFormProperties : public QWidget, public QgsExpress
           Container, //!< Container for the form
           QmlWidget,
           HtmlWidget,
-          WidgetType, //!< In the widget tree, the type of widget
-          Action, //!< Layer action
-          TextWidget, //!< Text widget type, \since QGIS 3.30
+          WidgetType,   //!< In the widget tree, the type of widget
+          Action,       //!< Layer action
+          TextWidget,   //!< Text widget type, \since QGIS 3.30
           SpacerWidget, //!< Spacer widget type, \since QGIS 3.30
         };
 
@@ -260,14 +263,47 @@ class GUI_EXPORT QgsAttributesFormProperties : public QWidget, public QgsExpress
          */
         void setCollapsedExpression( const QgsOptionalExpression &collapsedExpression );
 
+        /**
+         * Returns the relation editor configuration.
+         *
+         * \see setRelationEditorConfiguration()
+         */
         RelationEditorConfiguration relationEditorConfiguration() const;
-        void setRelationEditorConfiguration( RelationEditorConfiguration relationEditorConfiguration );
 
+        /**
+         * Sets the relation editor configuration.
+         *
+         * \see relationEditorConfiguration()
+         */
+        void setRelationEditorConfiguration( const RelationEditorConfiguration &relationEditorConfiguration );
+
+        /**
+         * Returns the QML editor configuration.
+         *
+         * \see setQmlElementEditorConfiguration()
+         */
         QmlElementEditorConfiguration qmlElementEditorConfiguration() const;
-        void setQmlElementEditorConfiguration( QmlElementEditorConfiguration qmlElementEditorConfiguration );
 
+        /**
+         * Sets the QML editor configuration.
+         *
+         * \see qmlElementEditorConfiguration()
+         */
+        void setQmlElementEditorConfiguration( const QmlElementEditorConfiguration &qmlElementEditorConfiguration );
+
+        /**
+         * Returns the HTML editor configuration.
+         *
+         * \see setHtmlElementEditorConfiguration()
+         */
         HtmlElementEditorConfiguration htmlElementEditorConfiguration() const;
-        void setHtmlElementEditorConfiguration( HtmlElementEditorConfiguration htmlElementEditorConfiguration );
+
+        /**
+         * Sets the HTML editor configuration.
+         *
+         * \see htmlElementEditorConfiguration()
+         */
+        void setHtmlElementEditorConfiguration( const HtmlElementEditorConfiguration &htmlElementEditorConfiguration );
 
         /**
          * Returns the spacer element configuration
@@ -323,23 +359,23 @@ class GUI_EXPORT QgsAttributesFormProperties : public QWidget, public QgsExpress
      */
     struct FieldConfig
     {
-      FieldConfig() = default;
-      FieldConfig( QgsVectorLayer *layer, int idx );
+        FieldConfig() = default;
+        FieldConfig( QgsVectorLayer *layer, int idx );
 
-      bool mEditable = true;
-      bool mLabelOnTop = false;
-      bool mReuseLastValues = false;
-      QgsFieldConstraints mFieldConstraints;
-      QPushButton *mButton = nullptr;
-      QString mEditorWidgetType;
-      QMap<QString, QVariant> mEditorWidgetConfig;
-      QString mAlias;
-      QgsPropertyCollection mDataDefinedProperties;
-      QString mComment;
-      Qgis::FieldDomainSplitPolicy mSplitPolicy = Qgis::FieldDomainSplitPolicy::Duplicate;
-      Qgis::FieldDuplicatePolicy mDuplicatePolicy = Qgis::FieldDuplicatePolicy::Duplicate;
+        bool mEditable = true;
+        bool mLabelOnTop = false;
+        bool mReuseLastValues = false;
+        QgsFieldConstraints mFieldConstraints;
+        QPushButton *mButton = nullptr;
+        QString mEditorWidgetType;
+        QMap<QString, QVariant> mEditorWidgetConfig;
+        QString mAlias;
+        QgsPropertyCollection mDataDefinedProperties;
+        QString mComment;
+        Qgis::FieldDomainSplitPolicy mSplitPolicy = Qgis::FieldDomainSplitPolicy::Duplicate;
+        Qgis::FieldDuplicatePolicy mDuplicatePolicy = Qgis::FieldDuplicatePolicy::Duplicate;
 
-      operator QVariant();
+        operator QVariant();
     };
 
   public:
@@ -399,6 +435,9 @@ class GUI_EXPORT QgsAttributesFormProperties : public QWidget, public QgsExpress
     void onAttributeSelectionChanged();
     void onFormLayoutSelectionChanged();
 
+    //! Context menu for Fields to enable Copy&Paste
+    void onContextMenuRequested( QPoint );
+
     void updatedFields();
 
   private:
@@ -409,26 +448,38 @@ class GUI_EXPORT QgsAttributesFormProperties : public QWidget, public QgsExpress
     void storeAttributeWidgetEdit();
 
     void loadAttributeTypeDialog();
-    void storeAttributeTypeDialog( );
+    void loadAttributeTypeDialogFromConfiguration( const FieldConfig &cfg );
+    void storeAttributeTypeDialog();
 
     void storeAttributeContainerEdit();
     void loadAttributeContainerEdit();
 
     void loadInfoWidget( const QString &infoText );
 
+    void copyWidgetConfiguration();
+    void pasteWidgetConfiguration();
+
     QTreeWidgetItem *loadAttributeEditorTreeItem( QgsAttributeEditorElement *widgetDef, QTreeWidgetItem *parent, QgsAttributesDnDTree *tree );
 
+    QgsMessageBar *mMessageBar = nullptr;
+
     Qgis::AttributeFormPythonInitCodeSource mInitCodeSource = Qgis::AttributeFormPythonInitCodeSource::NoSource;
     QString mInitFunction;
     QString mInitFilePath;
     QString mInitCode;
     int mBlockUpdates = 0;
 
+    //! Context menu for Fields
+    QMenu *mAvailableWidgetsTreeContextMenu = nullptr;
+    QAction *mActionCopyWidgetConfiguration = nullptr;
+    QAction *mActionPasteWidgetConfiguration = nullptr;
+
+    friend class TestQgsAttributesFormProperties;
 };
 
 
-QDataStream &operator<< ( QDataStream &stream, const QgsAttributesFormProperties::DnDTreeItemData &data );
-QDataStream &operator>> ( QDataStream &stream, QgsAttributesFormProperties::DnDTreeItemData &data );
+QDataStream &operator<<( QDataStream &stream, const QgsAttributesFormProperties::DnDTreeItemData &data );
+QDataStream &operator>>( QDataStream &stream, QgsAttributesFormProperties::DnDTreeItemData &data );
 
 
 /**
@@ -453,7 +504,7 @@ class GUI_EXPORT QgsAttributesDnDTree : public QTreeWidget, private QgsExpressio
      * Adds a new item to a \a parent. If \a index is -1, the item is added to the end of the parent's existing children.
      * Otherwise it is inserted at the specified \a index.
      */
-    QTreeWidgetItem *addItem( QTreeWidgetItem *parent, QgsAttributesFormProperties::DnDTreeItemData data, int index = -1, const QIcon &icon = QIcon() );
+    QTreeWidgetItem *addItem( QTreeWidgetItem *parent, const QgsAttributesFormProperties::DnDTreeItemData &data, int index = -1, const QIcon &icon = QIcon() );
 
     /**
      * Adds a new container to \a parent.
@@ -485,7 +536,7 @@ class GUI_EXPORT QgsAttributesDnDTree : public QTreeWidget, private QgsExpressio
   protected:
     QStringList mimeTypes() const override;
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+#if QT_VERSION < QT_VERSION_CHECK( 6, 0, 0 )
     QMimeData *mimeData( const QList<QTreeWidgetItem *> items ) const override;
 #else
     QMimeData *mimeData( const QList<QTreeWidgetItem *> &items ) const override;
